#!/usr/bin/python3

"""shortener submodule of pelican-shortener."""

from functools import wraps
import logging
import types

from pyshorteners.exceptions import ShorteningErrorException
from pyshorteners import Shortener

from pelican import signals

log = logging.getLogger(__name__)


def shortener_wrapper(generate_context, items_key="articles"):
    """Wrap generate_context."""
    s = Shortener()

    @wraps(generate_context)
    def wrapper(generator):
        result = generate_context()

        siteurl = generator.context["SITEURL"].rstrip("/")
        if not siteurl:
            log.info(f"Skipping URL shortening since SITEURL is empty")
            return result

        shorteners = sorted(s.available_shorteners, key=lambda x: (len(x), x))
        shorteners = generator.context.get("SHORTENERS", shorteners)
        if generator.context.get("PREFER_GITIO", True):
            try:
                shorteners.remove("gitio")
            except ValueError:
                pass
            shorteners = ["gitio"] + shorteners
        log.info(f"Shortening URLs with {shorteners}")

        for item in generator.__getattribute__(items_key):
            absolute_url = f"{siteurl}/{item.url}"

            i = 0
            while not hasattr(item, "short_url"):
                try:
                    item.short_url = getattr(s, shorteners[i]).short(absolute_url)
                except ShorteningErrorException:
                    pass
                i += 1

            log.info(f"Shortening URL {absolute_url} to {item.short_url}")
        return result

    return wrapper


def shorten_url_articles(generator):
    """Assign signal for shortening article URLs."""
    if generator.context.get("SHORTEN_ARTICLES", True):
        generator.generate_context = types.MethodType(
            shortener_wrapper(generator.generate_context), generator
        )


def shorten_url_pages(generator):
    """Assign signal for shortening page URLs."""
    if generator.context.get("SHORTEN_PAGES", True):
        generator.generate_context = types.MethodType(
            shortener_wrapper(generator.generate_context, items_key="pages"), generator
        )


def register():
    """Register plugin."""
    signals.article_generator_init.connect(shorten_url_articles)
    signals.page_generator_init.connect(shorten_url_pages)
